#pragma once
/*
Copyright (c) 2013-2017 Gustav Hartvigsson

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

/*
Adaptation of
http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/VERSIONS/C-LANG/mt19937-64.c
with the following license:

Copyright (C) 2004, Makoto Matsumoto and Takuji Nishimura,
All rights reserved.                          

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 3. The names of its contributors may not be used to endorse or promote 
    products derived from this software without specific prior written 
    permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/**
 * @file
 */


#include "defs.h"

S_BEGIN_DECLS

/**
 * @defgroup Random Random
 * @addtogroup Random
 * @{
 */

/**
 * An opaque data type that represents the a random value generator.
 *
 * SRandom is the perfered psuedo random number generator in libssts.
 * It is object oriented and as such can be very versitile.
 *
 * What it uses internally (right now) is an adapted version of mt19937-64 by
 * Makoto Matsumoto and Takuji Nishimura. (see:
 * http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/VERSIONS/C-LANG/mt19937-64.c)
 *
 * It will genrate a 64 bit random number. To use this 
 */
typedef struct SRandom SRandom;

S_EXPORTED
SRandom *
s_random_new ();

S_EXPORTED
void
s_random_free (SRandom * self);

S_EXPORTED
SRandom *
s_random_new_with_seed (sulong seed);

S_EXPORTED
void
s_random_reseed (SRandom * self, sulong seed);

S_EXPORTED
sulong
s_random_next (SRandom * self);

S_EXPORTED
sulong
s_random_current (SRandom * self);

/**
 * Generate a random number between @f$0@f$ and @f$2^{63}-1@f$
 */
S_EXPORTED
slong
s_random_next_long (SRandom * self);

/**
 * Generate a random real number in the interval @f$[0 ... 1]@f$.
 */
S_EXPORTED
sdouble
s_random_next_real (SRandom * self);

/**
 * Get the current random as a number between @f$0@f$ and @f$2^{63}-1@f$
 */
S_EXPORTED
slong
s_random_current_long (SRandom * self);

/**
 * Get the current random as a real number in the interval @f$[0 ... 1]@f$.
 */
S_EXPORTED
sdouble
s_random_current_real (SRandom * self);

#if 0
/* Get entropy from the system, how this is done under the hood is platform
 * specific.
 *
 * @note Do not abuse this as a source of random numbers, that will deplete the
 * entropy pool, which is not desirable.
 */
S_EXPORTED
slong
s_random_get_entropy ();
#endif

/**
 * @}
 */

S_END_DECLS
